from cms.api import add_plugin
from cms.test_utils.testcases import CMSTestCase

from djangocms_frontend.cms_plugins import (
    MyButtonPlugin,
    MyHeroPlugin,
    MyStrangeComponentPlugin,
)
from djangocms_frontend.contrib.alert.cms_plugins import AlertPlugin

from ..fixtures import TestFixture


class ComponentPluginTestCase(TestFixture, CMSTestCase):
    def test_component_with_empty_slots_plugin(self):
        instance = add_plugin(
            placeholder=self.placeholder,
            plugin_type=MyHeroPlugin.__name__,
            language=self.language,
        )
        instance.initialize_from_form(MyHeroPlugin.form).save()
        self.publish(self.page, self.language)

        with self.login_user_context(self.superuser):
            response = self.client.get(self.request_url)

        self.assertEqual(response.status_code, 200)
        # Default title is rendered in a formatted h1 tag
        self.assertInHTML(
            '<h1 class="max-w-2xl mb-4 text-4xl font-extrabold tracking-tight '
            'leading-none md:text-5xl xl:text-6xl dark:text-white">my title</h1>',
            response.content.decode("utf-8")
        )
        # Default slogan
        self.assertInHTML(
            '<p class="max-w-2xl mb-6 font-light text-gray-500 lg:mb-8 md:text-lg lg:text-xl '
            'dark:text-gray-400">'
            'django CMS\' plugins are great components'
            '</p>',
            response.content.decode("utf-8")
        )
        # Default slot content
        self.assertInHTML(
            '<a href="#" class="inline-flex items-center justify-center px-5 py-3 mr-3 '
            'text-base font-medium text-center text-white rounded-lg bg-primary-700 hover:bg-primary-800 '
            'focus:ring-4 focus:ring-primary-300 dark:focus:ring-primary-900">'
            'Get started'
            '<svg class="w-5 h-5 ml-2 -mr-1" fill="currentColor" viewBox="0 0 20 20" '
            'xmlns="http://www.w3.org/2000/svg"><path fill-rule="evenodd" '
            'd="M10.293 3.293a1 1 0 011.414 0l6 6a1 1 0 010 1.414l-6 6a1 1 0 01-1.414-1.414L14.586 11H3a1 '
            '1 0 110-2h11.586l-4.293-4.293a1 1 0 010-1.414z" clip-rule="evenodd"></path></svg>'
            '</a>',
            response.content.decode("utf-8")
        )

    def test_component_with_slots_plugin(self):
        from djangocms_frontend.cms_plugins import MyHeroSlotPlugin

        instance = add_plugin(
            placeholder=self.placeholder,
            plugin_type=MyHeroPlugin.__name__,
            language=self.language,
        )
        instance.initialize_from_form(MyHeroPlugin.form).save()
        slot = add_plugin(
            placeholder=self.placeholder,
            target=instance,
            plugin_type=MyHeroSlotPlugin.__name__,
            language=self.language,
        )
        add_plugin(
            placeholder=self.placeholder,
            target=slot,
            plugin_type=AlertPlugin.__name__,
            language=self.language,
        ).initialize_from_form(AlertPlugin.form).save()
        self.publish(self.page, self.language)

        with self.login_user_context(self.superuser):
            response = self.client.get(self.request_url)

        self.assertEqual(response.status_code, 200)

        # Default title is rendered in a formatted h1 tag
        self.assertInHTML(
            '<h1 class="max-w-2xl mb-4 text-4xl font-extrabold tracking-tight '
            'leading-none md:text-5xl xl:text-6xl dark:text-white">my title</h1>',
            response.content.decode("utf-8")
        )
        # Default slogan
        self.assertInHTML(
            '<p class="max-w-2xl mb-6 font-light text-gray-500 lg:mb-8 md:text-lg lg:text-xl '
            'dark:text-gray-400">'
            'django CMS\' plugins are great components'
            '</p>',
            response.content.decode("utf-8"),
            count=1
        )

        # Slot content is present
        self.assertInHTML(
            '<div class="alert alert-primary" role="alert"><div></div></div>',
            response.content.decode("utf-8"),
            count=1,
        )
        # Default slot content not present
        self.assertInHTML(
            '<a href="#" class="inline-flex items-center justify-center px-5 py-3 mr-3 '
            'text-base font-medium text-center text-white rounded-lg bg-primary-700 hover:bg-primary-800 '
            'focus:ring-4 focus:ring-primary-300 dark:focus:ring-primary-900">'
            'Get started'
            '<svg class="w-5 h-5 ml-2 -mr-1" fill="currentColor" viewBox="0 0 20 20" '
            'xmlns="http://www.w3.org/2000/svg"><path fill-rule="evenodd" '
            'd="M10.293 3.293a1 1 0 011.414 0l6 6a1 1 0 010 1.414l-6 6a1 1 0 01-1.414-1.414L14.586 11H3a1 '
            '1 0 110-2h11.586l-4.293-4.293a1 1 0 010-1.414z" clip-rule="evenodd"></path></svg>'
            '</a>',
            response.content.decode("utf-8"),
            count=0
        )

    def test_autocreate_slots(self):
        from djangocms_frontend.cms_plugins import MyHeroSlotPlugin, MyHeroTitlePlugin

        instance = add_plugin(
            placeholder=self.placeholder,
            plugin_type=MyHeroPlugin.__name__,
            language=self.language,
        )
        request = self.get_request(path="/")
        MyHeroPlugin().save_model(request=request, obj=instance, form=MyHeroPlugin.form, change=False)

        plugins = list(self.placeholder.get_plugins())

        self.assertEqual(len(plugins), 3)
        self.assertEqual(plugins[0].plugin_type, MyHeroPlugin.__name__)
        self.assertIsNone(plugins[0].parent)
        self.assertEqual(plugins[1].plugin_type, MyHeroTitlePlugin.__name__)
        self.assertEqual(plugins[1].parent.pk, instance.pk)
        self.assertEqual(plugins[2].plugin_type, MyHeroSlotPlugin.__name__)
        self.assertEqual(plugins[2].parent.pk, instance.pk)

    def test_simple_component_plugin(self):
        instance = add_plugin(
            placeholder=self.placeholder,
            plugin_type=MyButtonPlugin.__name__,
            language=self.language,
        )
        instance.initialize_from_form(MyButtonPlugin.form)
        instance.config["link"] = {"internal_link": f"cms.page:{self.page.pk}"}
        instance.save()

        from djangocms_link.templatetags.djangocms_link_tags import to_url
        link = to_url(instance.link)

        self.publish(self.page, self.language)

        with self.login_user_context(self.superuser):
            response = self.client.get(self.request_url)

        self.assertEqual(response.status_code, 200)
        self.assertInHTML(f'<a class="btn" href="{link}">Click me</a>', response.content.decode("utf-8"))

    def test_strange_plugin(self):
        self.assertEqual(MyStrangeComponentPlugin.name, "MyStrangeComponent")
        self.assertEqual(MyStrangeComponentPlugin.render_template, "djangocms_frontend/html_container.html")
        self.assertFalse(MyStrangeComponentPlugin.allow_children)



from cms.plugin_pool import plugin_pool
from django.utils.translation import gettext_lazy as _

from ... import settings
from ...cms_plugins import CMSUIPlugin
from ...common import AttributesMixin, BackgroundMixin, ResponsiveMixin, SpacingMixin
from .. import icon
from . import forms, models

mixin_factory = settings.get_renderer(icon)


class IconPlugin(
    mixin_factory("Icon"),
    AttributesMixin,
    ResponsiveMixin,
    SpacingMixin,
    BackgroundMixin,
    CMSUIPlugin,
):
    """
    Universal icon picker
    https://github.com/migliori/universal-icon-picker
    """

    name = _("Icon")
    module = _("Frontend")
    model = models.Icon
    form = forms.IconForm
    text_enabled = True
    text_icon = (
        '<svg xmlns="http://www.w3.org/2000/svg" width="16" height="16" fill="currentColor" '
        'class="bi bi-emoji-sunglasses" viewBox="0 0 16 16"><path d="M4.968 9.75a.5.5 0 1 0-.866.5A4.5 4.5 '
        "0 0 0 8 12.5a4.5 4.5 0 0 0 3.898-2.25.5.5 0 1 0-.866-.5A3.5 3.5 0 0 1 8 11.5a3.5 3.5 0 0 1-3.032-1.75M7 "
        "5.116V5a1 1 0 0 0-1-1H3.28a1 1 0 0 0-.97 1.243l.311 1.242A2 2 0 0 0 4.561 8H5a2 2 0 0 0 1.994-1.839A3 "
        "3 0 0 1 8 6c.393 0 .74.064 1.006.161A2 2 0 0 0 11 8h.438a2 2 0 0 0 1.94-1.515l.311-1.242A1 1 0 0 0 "
        '12.72 4H10a1 1 0 0 0-1 1v.116A4.2 4.2 0 0 0 8 5c-.35 0-.69.04-1 .116"/><path d="M16 8A8 8 0 1 1 0 8a8 8 '
        '0 0 1 16 0m-1 0A7 7 0 1 0 1 8a7 7 0 0 0 14 0"/></svg>'
    )
    fieldsets = [
        (
            None,
            {
                "fields": (
                    (
                        "icon",
                        "icon_size",
                    ),
                    "icon_foreground",
                    "icon_rounded",
                )
            },
        ),
    ]


if "IconPlugin" in plugin_pool.plugins:
    #  Unregister already installed IconPlugin
    del plugin_pool.plugins["IconPlugin"]

plugin_pool.register_plugin(IconPlugin)



# Generated by Django 5.0.6 on 2024-05-22 20:33

from django.db import migrations


def rename(apps, old_name, new_name):
    UIItem = apps.get_model("djangocms_frontend", "FrontendUIItem")
    UIItem.objects.filter(ui_item=old_name).update(ui_item=new_name, plugin_type=f"{new_name}Plugin")


class Migration(migrations.Migration):

    dependencies = [
        ('link', '0001_initial'),
    ]

    operations = [
        migrations.RunPython(
            lambda apps, schema_editor: rename(apps, "Link", "TextLink"),
            lambda apps, schema_editor: rename(apps, "TextLink", "Link"),
            elidable=True,
        ),
    ]



from django.test.testcases import TestCase


class LinkMigrationTestCase(TestCase):
    def setUp(self):
        import importlib

        link_migration = importlib.import_module("djangocms_frontend.migrations.0002_migrate_links")
        self.convert = staticmethod(link_migration.convert_item)

    def test_link_migration(self):
        test_links = (
            dict(external_link="https://www.django-cms.com"),
            dict(external_link="https://www.django-cms.com", anchor="top"),
            dict(internal_link=dict(model="cms.page", pk=1)),
            dict(internal_link=dict(model="cms.page", pk=1), anchor="top"),
            dict(file_link=dict(model="filer.file", pk=1)),
            dict(phone="1234567890"),
            dict(mailto="mail@example.com")
        )

        for link in test_links:
            with self.subTest(link=link):
                config = link.copy()
                self.convert(config, "forward")
                self.convert(config, "backward")
                self.assertEqual(link, config)



from djangocms_frontend.helpers import is_first_child


class ImageRenderMixin:
    def render(self, context, instance, placeholder):
        if instance.alignment:
            # See https://getbootstrap.com/docs/5.2/content/images/#aligning-images
            if instance.alignment != "center":
                instance.add_classes(f"float-{instance.alignment}")
            else:
                instance.add_classes("mx-auto d-block")
        if instance.picture_fluid:
            instance.add_classes("img-fluid")
        if instance.picture_rounded:
            instance.add_classes("rounded")
        if instance.picture_thumbnail:
            instance.add_classes("img-thumbnail")
        if instance.parent and instance.parent.plugin_type == "CardPlugin":
            instance.add_classes("card-img-top" if is_first_child(instance, instance.parent) else "card-img-bottom")
        elif instance.parent and instance.parent.plugin_type == "FigurePlugin":
            instance.add_classes("figure-img")
        return super().render(context, instance, placeholder)



import warnings

from django.utils.module_loading import autodiscover_modules


class Components:
    _registry: dict = {}
    _discovered: bool = False

    def register(self, component):
        if component.__name__ in self._registry:
            warnings.warn(f"Component {component.__name__} already registered", stacklevel=2)
            return component
        self._registry[component.__name__] = component.get_registration()
        return component

    def __getitem__(self, item):
        return self._registry[item]


components = Components()

if not components._discovered:
    autodiscover_modules("cms_components", register_to=components)
    components._discovered = True



from django import forms
from djangocms_link.fields import LinkFormField

from djangocms_frontend.component_base import CMSFrontendComponent
from djangocms_frontend.component_pool import components
from djangocms_frontend.contrib.image.fields import ImageFormField


@components.register
class MyHero(CMSFrontendComponent):
    class Meta:
        # declare plugin properties
        name = "My Hero Component"
        render_template = "hero.html"
        allow_children = True
        mixins = ["Background"]
        slots = (
            ("title", "Title"),
            ("slot", "Slot"),
        )
        # for more complex components, you can add fieldsets

    # declare fields
    title = forms.CharField(required=True, initial="my title")
    slogan = forms.CharField(required=True, initial="django CMS' plugins are great components", widget=forms.Textarea)
    image = ImageFormField(required=True)

    # add description for the structure board
    def get_short_description(self):
        return self.title


@components.register
class MyButton(CMSFrontendComponent):
    class Meta:
        name = "Button"
        render_template = "button.html"
        allow_children = False

    text = forms.CharField(required=True, initial="Click me")
    link = LinkFormField()

    def get_short_description(self):
        return self.text


@components.register
class MyStrangeComponent(CMSFrontendComponent):
    text = forms.CharField(required=True, initial="Message")



from django import apps
from django.core import checks


class DjangocmsFrontendConfig(apps.AppConfig):
    name = "djangocms_frontend"
    verbose_name = "django CMS Frontend"

    def ready(self):
        from . import plugin_tag

        plugin_tag.setup()
        checks.register(check_settings)
        checks.register(check_installed_apps)


def check_settings(*args, **kwargs):  # pragma: no cover
    from django.conf import settings

    warnings = []

    if hasattr(settings, "DJANGOCMS_FRONTEND_MINIMUM_INPUT_LENGTH"):
        warnings.append(
            checks.Warning(
                "The DJANGOCMS_FRONTEND_MINIMUM_INPUT_LENGTH setting was removed in djangocms-frontend 2.",
                "Use DJANGOCMS_LINK_MINIMUM_INPUT_LENGTH instead.",
                id="djangocms_frontend.W001",
                obj="settings.DJANGOCMS_FRONTEND_MINIMUM_INPUT_LENGTH",
            )
        )
    if hasattr(settings, "DJANGOCMS_FRONTEND_LINK_MODELS"):
        warnings.append(
            checks.Warning(
                "The DJANGOCMS_FRONTEND_LINK_MODELS setting was removed in djangocms-frontend 2. "
                "djangocms-frontend 2 uses linkable models from djangocms-link. See "
                "https://github.com/django-cms/djangocms-link#django-cms-link for more info.",
                "This message disappears after removing the DJANGOCMS_FRONTEND_LINK_MODELS from your "
                "project's settings.\n",
                id="djangocms_frontend.W002",
                obj="settings.DJANGOCMS_FRONTEND_LINK_MODELS",
            )
        )
    return warnings


def check_installed_apps(*args, **kwargs):  # pragma: no cover
    from django.conf import settings

    errors = []
    link_contrib_apps = [
        "djangocms_frontend.contrib.carousel",
        "djangocms_frontend.contrib.image",
        "djangocms_frontend.contrib.link",
    ]
    link_apps_used = [app for app in link_contrib_apps if app in settings.INSTALLED_APPS]
    if link_apps_used:
        if "djangocms_link" not in settings.INSTALLED_APPS:
            errors.append(
                checks.Error(
                    "djangocms-frontend requires djangocms-link to be installed for {}.".format(
                        ", ".join(link_apps_used)
                    ),
                    "Add 'djangocms_link' to your INSTALLED_APPS setting or remove all of the above apps.",
                    id="djangocms_frontend.E001",
                    obj="settings.INSTALLED_APPS",
                )
            )
    return errors



import importlib
import typing

from cms.api import add_plugin
from cms.plugin_base import CMSPluginBase
from django import forms
from django.apps import apps
from django.utils.encoding import force_str
from django.utils.translation import gettext_lazy as _
from entangled.forms import EntangledModelForm

from .ui_plugin_base import CMSUIComponent


def _import_or_empty(module, name):
    try:
        return importlib.import_module(module).__dict__[name]
    except (ImportError, KeyError):
        return type(name, (), {})


def _get_mixin_classes(mixins: list, suffix: str = "") -> list[type]:
    """Find and import mixin classes from a list of mixin strings"""
    mixins = [
        (mixin.rsplit(".")[0], f"{mixin.rsplit('.')[-1]}{suffix}Mixin")
        if "." in mixin
        else ("djangocms_frontend.common", f"{mixin}{suffix}Mixin")
        for mixin in mixins
    ]

    return [_import_or_empty(module, name) for module, name in mixins]


class Slot:
    """Slat class as syntactic surgar to more easily define slot plugins"""

    def __init__(self, name, verbose_name, **kwargs):
        self.name = name
        self.verbose_name = verbose_name
        self.kwargs = kwargs


class CMSFrontendComponent(forms.Form):
    """Base class for frontend components:"""

    slot_template = "djangocms_frontend/slot.html"
    _base_form = EntangledModelForm
    _plugin_mixins = []
    _model_mixins = []
    _admin_form = None
    _model = None
    _plugin = None
    META_FIELDS = [
        "is_local",
        "disable_edit",
        "disable_child_plugins",
        "show_add_form",
        "frontend_editable_fields",
        "link_fieldset_position",
        "require_parent",
        "parent_classes",
    ]

    @classmethod
    def admin_form_factory(cls, **kwargs) -> type:
        if cls._admin_form is None:
            from djangocms_frontend.models import FrontendUIItem

            mixins = getattr(cls._component_meta, "mixins", [])
            mixins = _get_mixin_classes(mixins, "Form")
            cls._admin_form = type(
                f"{cls.__name__}Form",
                (
                    *mixins,
                    cls,
                    cls._base_form,
                ),
                {
                    **kwargs,
                    "Meta": type(
                        "Meta",
                        (),
                        {
                            "model": FrontendUIItem,
                            "entangled_fields": {
                                "config": list(cls.declared_fields.keys()),
                            },
                        },
                    ),
                },
            )
        return cls._admin_form

    @classmethod
    def get_slot_plugins(cls) -> dict[str:str]:
        slots: list[Slot] = [
            slot if isinstance(slot, Slot) else Slot(*slot) for slot in getattr(cls._component_meta, "slots", [])
        ]
        return {f"{cls.__name__}{slot.name.capitalize()}Plugin": slot for slot in slots}

    @classmethod
    def plugin_model_factory(cls) -> type:
        if cls._model is None:
            from djangocms_frontend.models import FrontendUIItem

            app_config = apps.get_containing_app_config(cls.__module__)
            if app_config is None:
                raise ValueError(f"Cannot find app_config for {cls.__module__}")
            cls._model = type(
                cls.__name__,
                (
                    *cls._model_mixins,
                    FrontendUIItem,
                ),
                {
                    "Meta": type(
                        "Meta",
                        (),
                        {
                            "app_label": app_config.label,
                            "proxy": True,
                            "managed": False,
                            "verbose_name": getattr(cls._component_meta, "name", cls.__name__),
                        },
                    ),
                    "get_short_description": cls.get_short_description,
                    "__module__": "djangocms_frontend.contrib.component.models",
                },
            )
        return cls._model

    @classmethod
    def plugin_factory(cls) -> type:
        if cls._plugin is None:
            mixins = getattr(cls._component_meta, "mixins", [])
            slots = cls.get_slot_plugins()
            mixins = _get_mixin_classes(mixins)

            cls._plugin = type(
                cls.__name__ + "Plugin",
                (
                    *mixins,
                    *cls._plugin_mixins,
                    CMSUIComponent,
                ),
                {
                    "name": getattr(cls._component_meta, "name", cls.__name__),
                    "module": getattr(cls._component_meta, "module", _("Components")),
                    "model": cls.plugin_model_factory(),
                    "form": cls.admin_form_factory(),
                    "allow_children": slots or getattr(cls._component_meta, "allow_children", False),
                    "child_classes": getattr(cls._component_meta, "child_classes", []) + list(slots.keys()),
                    "render_template": getattr(cls._component_meta, "render_template", CMSUIComponent.render_template),
                    "fieldsets": getattr(cls, "fieldsets", cls._generate_fieldset()),
                    "change_form_template": "djangocms_frontend/admin/base.html",
                    "slots": slots,
                    "save_model": cls.save_model,
                    **{
                        field: getattr(cls._component_meta, field)
                        for field in cls.META_FIELDS
                        if hasattr(cls._component_meta, field)
                    },
                    **(
                        {
                            "get_render_template": cls.get_render_template,
                            "TEMPLATES": cls.TEMPLATES,
                        }
                        if hasattr(cls, "get_render_template")
                        else {}
                    ),
                },
            )
        return cls._plugin

    @classmethod
    def slot_plugin_factory(cls) -> list[type]:
        slots = cls.get_slot_plugins()
        return [
            type(
                name,
                (CMSPluginBase,),
                {
                    "name": force_str(slot.verbose_name),
                    "module": getattr(cls._component_meta, "module", _("Component")),
                    "allow_children": True,
                    "edit_disabled": True,
                    "parent_classes": cls.__name__ + "Plugin",
                    "render_template": cls.slot_template,
                    **slot.kwargs,
                },
            )
            for name, slot in slots.items()
        ]

    @classmethod
    def get_registration(cls) -> tuple[type, type, list[type]]:
        return (
            cls.plugin_model_factory(),
            cls.plugin_factory(),
            cls.slot_plugin_factory(),
        )

    @classmethod
    @property
    def _component_meta(cls) -> typing.Optional[type]:
        return getattr(cls, "Meta", None)

    @classmethod
    def _generate_fieldset(cls) -> list[tuple[typing.Optional[str], dict]]:
        return [(None, {"fields": cls.declared_fields.keys()})]

    def get_short_description(self) -> str:
        return self.config.get("title", "")

    def save_model(self, request, obj, form: forms.Form, change: bool) -> None:
        """Auto-creates slot plugins upon creation of component plugin instance"""

        super(CMSUIComponent, self).save_model(request, obj, form, change)
        if not change:
            for slot in self.slots.keys():
                add_plugin(obj.placeholder, slot, obj.language, target=obj)



from django.db.models import ManyToOneRel
from filer.fields.image import AdminImageFormField, FilerImageField
from filer.models import Image


class ImageFormField(AdminImageFormField):
    def __init__(self, *args, **kwargs):
        kwargs.setdefault("rel", ManyToOneRel(FilerImageField, Image, "id"))
        kwargs.setdefault("queryset", Image.objects.all())
        kwargs.setdefault("to_field_name", "id")
        super().__init__(*args, **kwargs)



import uuid

from cms.models import CMSPlugin
from django.core.serializers.json import DjangoJSONEncoder
from django.db import models
from django.utils.html import conditional_escape, mark_safe
from django.utils.translation import gettext
from django.utils.translation import gettext_lazy as _

from djangocms_frontend.fields import TagTypeField
from djangocms_frontend.settings import FRAMEWORK_PLUGIN_INFO


class AbstractFrontendUIItem(CMSPlugin):
    """
    The `AbstractFrontendUIItem` class is an abstract base class that provides common functionality
    for frontend UI items in a CMS plugin. It is a subclass of `CMSPlugin` class.

    Use this class as a base class for custom plugins that add their own database fields.

    Attributes:
        - ui_item: A CharField that represents the UI item name (max length 30).
        - tag_type: A TagTypeField (custom field) that represents the type of HTML tag for the UI item.
        - config: A JSONField that stores additional configuration for the UI item.

    Methods:
        - __init__(*args, **kwargs): Constructor method that initializes the object and sets additional classes.
        - __getattr__(item): Allows properties of the plugin config to be accessed as plugin properties.
        - __str__(): Returns a string representation of the UI item.
        - add_classes(*args): Adds additional classes to the UI item.
        - add_attribute(attr, value=None): Adds an attribute to the configuration attributes.
        - get_attributes(): Returns the attributes as a string for rendering the UI item.
        - save(*args, **kwargs): Saves the UI item to the database.
        - initialize_from_form(form=None): Populates the config JSON field based on initial values from a form.
        - get_short_description(): Returns a plugin-specific short description.
        - framework_info: Returns the framework information for the UI item.

    Note: This is an abstract base class and should not be used directly.
    """

    class Meta:
        abstract = True
        verbose_name = _("UI item")

    ui_item = models.CharField(max_length=30)
    tag_type = TagTypeField(blank=True)
    config = models.JSONField(default=dict, encoder=DjangoJSONEncoder)

    def __init__(self, *args, **kwargs):
        self._additional_classes = []
        self.uuid = str(uuid.uuid4())
        super().__init__(*args, **kwargs)

    def __getattr__(self, item):
        """Makes properties of plugin config available as plugin properties."""
        if item[0] != "_" and item in self.config:  # Avoid infinite recursion trying to get .config from db
            return self.config[item]
        return super().__getattribute__(item)

    def __str__(self):
        if "__str__" in self.config:
            return self.config["__str__"]
        return f"{gettext(self.ui_item)} ({str(self.pk)})"

    def add_classes(self, *args):
        for arg in args:
            if arg:
                self._additional_classes += arg.split() if isinstance(arg, str) else arg

    def add_attribute(self, attr, value=None):
        attrs = self.config.get("attributes", {})
        if attr == "style" and attr in attrs:
            value += attrs[attr]
        attrs.update({attr: value})
        self.config["attributes"] = attrs

    def get_attributes(self):
        attributes = self.config.get("attributes", {})
        classes = set(attributes.get("class", "").split())  # classes added in attriutes
        classes.update(self._additional_classes)  # add additional classes
        classes = (f'class="{conditional_escape(" ".join(classes))}"') if classes else ""  # to string
        parts = (
            f'{item}="{conditional_escape(value)}"' if value else f"{item}"
            for item, value in attributes.items()
            if item != "class"
        )
        attributes_string = (classes + " ".join(parts)).strip()
        return mark_safe(" " + attributes_string) if attributes_string else ""

    def save(self, *args, **kwargs):
        self.ui_item = self.__class__.__name__
        return super().save(*args, **kwargs)

    def initialize_from_form(self, form=None):
        """Populates the config JSON field based on initial values provided by the fields of form"""
        if form is None:
            form = self.get_plugin_class().form
        if isinstance(form, type):  # if is class
            if not getattr(form._meta, "model", None):
                form._meta.model = self.__class__
            form = form()  # instantiate
        entangled_fields = getattr(getattr(form, "_meta", None), "entangled_fields", {}).get("config", ())
        for field in entangled_fields:
            self.config.setdefault(field, {} if field == "attributes" else form[field].initial or "")
        return self

    def get_short_description(self):
        """Plugin-specific short description (to be defined by subclasses). Try title attribute first."""
        return self.config.get("title", "")

    @property
    def framework_info(self):
        return FRAMEWORK_PLUGIN_INFO.get(self.__class__.__name__, None)


class FrontendUIItem(AbstractFrontendUIItem):
    """

    Class: FrontendUIItem

    Inherits From: AbstractFrontendUIItem

    Description:
    This class represents a UI item in the frontend. It is used to define the behavior and attributes
    of a UI item in the user interface.

    Use this class as a base class for custom plugins that do not add their own database fields but
    use the entangled form fields instead.

    Attributes:
    - verbose_name (str): The verbose name of the UI item.

    Methods:
    None

    """

    class Meta:
        verbose_name = _("UI item")



from cms.plugin_pool import plugin_pool

from .component_pool import components
from .ui_plugin_base import CMSUIPluginBase


class CMSUIPlugin(CMSUIPluginBase):
    pass


# Loop through the values in the components' registry
for _, plugin, slot_plugins in components._registry.values():
    # Add the plugin to the global namespace
    globals()[plugin.__name__] = plugin
    # Register the plugin with the plugin pool
    plugin_pool.register_plugin(plugin)

    # Loop through the slot plugins associated with the current plugin
    for slot_plugin in slot_plugins:
        # Add the slot plugin to the global namespace
        globals()[slot_plugin.__name__] = slot_plugin
        # Register the slot plugin with the plugin pool
        plugin_pool.register_plugin(slot_plugin)



from django.core.files.base import ContentFile

from djangocms_frontend.contrib.image.cms_plugins import ImagePlugin
from djangocms_frontend.contrib.image.forms import get_templates
from djangocms_frontend.contrib.image.models import Image
from djangocms_frontend.helpers import add_plugin, first_choice

default_template = first_choice(get_templates())


def create_image_plugin(filename, file, parent_plugin, **kwargs):
    # Set the FilerImageField value.
    from filer.settings import FILER_IMAGE_MODEL
    from filer.utils.loader import load_model

    image_class = load_model(FILER_IMAGE_MODEL)
    image_obj = image_class(file=ContentFile(file.read(), name=filename))
    image_obj.save()

    img = Image(
        parent=parent_plugin,
        position=parent_plugin.position + 1,
        placeholder=parent_plugin.placeholder,
        language=parent_plugin.language,
        plugin_type=ImagePlugin.__name__,
        ui_item=Image.__class__.__name__,
        config={},
    ).initialize_from_form()
    img.config.update(
        {
            "picture": {"pk": image_obj.pk, "model": "filer.image"},
            "use_no_cropping": True,
        }
    )
    add_plugin(parent_plugin.placeholder, img)

    return img
